'-------------------------------------------------
'		sr2pegtool graphics
'		Copyright (c) 2018 Masamichi Kirikae
'-------------------------------------------------
#include "graphics.bi"

namespace graphics

' convert rgba color to rgb color with white background. 
function rgba_to_rgb(c as rgba_data) as rgba_data
	dim t as double, tc as rgba_data
	t = c.r * (c.a / 255.0) + 255.0 * (1.0 - c.a / 255.0): if (t > 255.0) then t = 255.0
	tc.r = cubyte(t)
	t = c.g * (c.a / 255.0) + 255.0 * (1.0 - c.a / 255.0): if (t > 255.0) then t = 255.0
	tc.g = cubyte(t)
	t = c.b * (c.a / 255.0) + 255.0 * (1.0 - c.a / 255.0): if (t > 255.0) then t = 255.0
	tc.b = cubyte(t)
	return tc	
end function

'	draw background
sub draw_background(xofs as s32, yofs as s32, imagew as u32, imageh as u32, bktype as s32)
	dim as s32 c, x, y, xmax, ymax, nx, ny
	xmax = xofs + imagew - 1
	ymax = yofs + imageh - 1
	select case bktype
		case BACKGROUND_GRID
			c = rgb(153, 153, 153)
			line (xofs, yofs) - (xmax, ymax), c, BF
			ny = 1
			for y = yofs to ymax step 8
				ny = (not ny) + 1
				nx = ny
				for x = xofs to xmax step 8
					nx = (not nx) + 1
					if (nx > 0) then	'dark gray
						c = rgb(102, 102, 102)
						line (x, y) - (x+7, y+7), c, BF
					else
						c = rgb(153, 153, 153)
						line (x, y) - (x+7, y+7), c, BF
					end if
				next
			next
		case BACKGROUND_WHITE
			c = rgb(255, 255, 255)
			line (xofs, yofs) - (xmax, ymax), c, BF
		case BACKGROUND_BLACK
			c = rgb(0, 0, 0)
			line (xofs, yofs) - (xmax, ymax), c, BF
		case else
	end select
end sub

sub draw_image(title as string, image_rgba() as u8, w as u32, h as u32)
	dim as s32 n, x, y, c, rwidth, rheight, xofs, yofs
	dim crgba as rgba_data
	dim imgbuf as any ptr

	if (w < 256) then 
		rwidth = 256
		xofs = (256 - w) \ 2 - 1
	else
		rwidth = w
		xofs = 0
	endif
	if (h < 256) then
		rheight = 256
		yofs = (256 - h) \ 2 - 1
	else
		rheight = h
		yofs = 0
	endif

	screenres(rwidth, rheight, 32)
	windowtitle(title)

	c = rgb(255, 255, 255)
	line (0, 0) - (rwidth - 1, rheight - 1), c, BF
	draw_background(xofs, yofs, w, h, BACKGROUND_GRID)

	imgbuf = imagecreate(w, h, 0)
	n = 0
	for y = 0 to h - 1
		for x = 0 to w - 1
			crgba.r = image_rgba(n): n+=1
			crgba.g = image_rgba(n): n+=1
			crgba.b = image_rgba(n): n+=1
			crgba.a = image_rgba(n): n+=1
			c = rgba(crgba.r, crgba.g, crgba.b, crgba.a)
			pset imgbuf, (x, y), c
		next
	next
	' draw image on background with alpha
	put (xofs, yofs), imgbuf, alpha
	' release image buffer memory
	imagedestroy(imgbuf)
end sub

sub close_window()
	if (screenptr() <> 0) then
		screen 0
	endif
end sub

' draw dds image with all mipmaps
sub draw_dds_blocks(title as string, image() as u8, w as u32, h as u32, fmt as s32, mipmap as s32)
	dim as s32 rimgw, rimgh, xofs, yofs
	dim as s32 imgw, imgh, imagew, imageh, imagesize, offset, flags
	dim as s32 n, i, x, y, sx, sy, r, g, b, a, c
	dim as s32 ret, mousebutton_stat, last_mousebutton_stat
	dim mouse as mouse_data
	dim image_rgba(any) as u8
	dim imgbuf as any ptr
	dim as s32 bktype = BACKGROUND_GRID
	dim as string kstr

	if (mipmap > 1) then
		imgw = w + w \ 2
		imgh = h
	else
		imgw = w
		imgh = h
	end if

	' minimum window size = 512x512
	if (imgw < 512) then
		rimgw = 512
		xofs = (512 - imgw) \ 2
	else
		rimgw = imgw
		xofs = 0
	endif
	if (imgh < 512) then
		rimgh = 512
		yofs = (512 - imgh) \ 2
	else
		rimgh = imgh
		yofs = 0
	endif

	' open graphic windows
	screenres(rimgw, rimgh, 32)
	windowtitle(title)
	' draw white background
	draw_background(0, 0, rimgw, rimgh, BACKGROUND_WHITE)
	draw_background(xofs, yofs, imgw, imgh, BACKGROUND_GRID)
	imgbuf = imagecreate(imgw, imgh, 0)

	' set DXTC flags
	flags = 0
	select case fmt
		case D3DFMT_DXT1
			flags = squish_kDxt1
		case D3DFMT_DXT3
			flags = squish_kDxt3
		case D3DFMT_DXT5
			flags = squish_kDxt5
		case else
	end select

	if (flags) then
		imagew = w
		imageh = h
		
		imagesize = imagew * 2 * imageh * 4
		redim image_rgba(imagesize - 1) as u8
		
		imagesize = imagew * imageh * 4
		n = 0
		offset = 0
		for i = 1 to mipmap
			squish.DecompressImage(@image_rgba(n), imagew, imageh, @image(offset), flags)
			offset += squish.GetstorageRequirements(imagew, imageh, flags)
			n += imagesize
			imagew /= 2
			imageh /= 2
			imagesize = imagew * imageh * 4
		next
	endif

	' draw image to buffer
	select case fmt
		case D3DFMT_A8R8G8B8, D3DFMT_R5G6B5
			imagesize = imgw * imgh * 4
			redim image_rgba(imagesize - 1) as u8
			n = 0
			for y = 0 to imgh - 1
				for x = 0 to imgw - 1
					r = image(n): n+=1
					g = image(n): n+=1
					b = image(n): n+=1
					a = image(n): n+=1
					c = rgba(r, g, b, a)
					pset imgbuf, (x, y), c
				next
			next
		case D3DFMT_DXT1, D3DFMT_DXT3, D3DFMT_DXT5
			n = 0
			for i = 0 to mipmap - 1
				select case i
					case 0
						sx = 0
						sy = 0
						imagew = w
						imageh = h
					case 1
						sx = w
						sy = 0
						imagew = w \ 2
						imageh = h \ 2
					case is > 1
						sy = sy + imgh\(2^(i-1))
						imagew = w \ (2^i)
						imageh = h \ (2^i)
					case else
				end select
				'	draw mipmap
				for y = sy to sy + imageh - 1
					for x = sx to sx + imagew - 1
						r = image_rgba(n): n+=1
						g = image_rgba(n): n+=1
						b = image_rgba(n): n+=1
						a = image_rgba(n): n+=1
						c = rgba(r, g, b, a)
						pset imgbuf, (x, y), c
					next
				next
			next
		case else
	end select

	do
		' draw image on background with alpha
		put (xofs, yofs), imgbuf, alpha
		
		' key or mouse click wait
		do
			ret = getmouse(mouse.x, mouse.y, mouse.wheel, mouse.button, mouse.clip)
			if (ret = 0) then	'	mouse cursor on graphics window
				mousebutton_stat = mouse.button and &H00000001	'	check mouse left button status
				if (last_mousebutton_stat = 0) then 
					if (mousebutton_stat = 1) then
						last_mousebutton_stat = 1
						exit do
					endif
				else
					if (mousebutton_stat = 0) then
						last_mousebutton_stat = 0
					endif
				endif
			endif
			kstr = inkey()
		loop while (kstr = "")

		if ((kstr = GRAPHICS_WINDOW_CLOSED) or (kstr = KEY_ESC)) then
			' release image buffer memory
			imagedestroy(imgbuf)
			graphics.close_window()
			exit do
		else	' change background type
			bktype = bktype shl 1
			if (bktype = BACKGROUND_END) then bktype = BACKGROUND_GRID
			select case bktype
				case BACKGROUND_GRID
					draw_background(xofs, yofs, imgw, imgh, BACKGROUND_GRID)
				case BACKGROUND_WHITE
					draw_background(xofs, yofs, imgw, imgh, BACKGROUND_WHITE)
				case BACKGROUND_BLACK
					draw_background(xofs, yofs, imgw, imgh, BACKGROUND_BLACK)
				case else
			end select
		endif		
	loop while (true)
end sub

end namespace
