'-------------------------------------------------
'		sr2pegtool Saints Row 2 Texture Tool
'		Copyright (c) 2016 Masamichi Kirikae
'-------------------------------------------------
#include once "sr2pegtool.bi"

'----------------------------------------------------------
'	main
'----------------------------------------------------------
function main() as s32
	dim as s32 n
	dim as string srcfilename, dstfilename, extname, kstr, currentdir
	dim as double dk, start_time, end_time

	currentdir = curdir
	chdir(exepath)
	
	srcfilename = command(1)

	'	open console handle
	console = freefile
	open cons for output as #console

	if (dir(srcfilename, fbArchive) = "") then
		cprint "sr2pegtool saints row 2 texture tool version "; PROGRAM_VERSION 
		cprint ""
		cprint "Usage: sr2pegtool.exe <filename> or,"
		cprint "Drag a file and drop it on sr2pegtool.exe"
		cprint ""
		cprint " Supported files"
		cprint "*.peg_pc   file (extract package)"
		cprint "*.peg_desc file (rebuild package)"
		cprint "*.g_peg_pc file (texture viewer)"
		cprint "*.dds      file (dds file viewer)"
		cprint "*.png      file (convert png to dds or normalmap to tgn)"
		cprint ""
		chdir(currentdir)
		exit_main(F_ERROR)
	end if

	' initialize dds director
	dds.init_dds()

	' initialize peg director
	peg_pc.init_peg()

	'	read ini file
	dstfilename = exepath + "\" + _INIFILE_NAME
	n = read_ini(dstfilename)
	if (n = F_ERROR) then
		chdir(currentdir)
		exit_main(n)
	endif
	
	if (dds.dds_director.silentmode = false) then
		cprint "sr2pegtool.exe version "; PROGRAM_VERSION
		cprint ""
	endif

	extname = libfio.get_filename_ext(srcfilename)
	select case lcase(extname) 
		case "peg_pc"
			n = peg_pc.extract_peg(srcfilename, "")
			if (n = F_ERROR) then
				cprint "Error: Fail to extract."
				chdir(currentdir)
				exit_main(n)
			endif
		case "peg_desc"
			dim as string method, colourspace
			dim as double  sharp

			method = dds.dds_director.mipmap(DDS_MIPMAP_DXT1).resize_filter
			colourspace = get_image_colour_string(dds.dds_director.mipmap(DDS_MIPMAP_DXT1).colourspace)
			cprint       "> Texture    Resize Filter              Color "
			cprint using "> DXT1       \                        \ \    \"; method; colourspace
		
			method = dds.dds_director.mipmap(DDS_MIPMAP_DXT3_DXT5).resize_filter
			colourspace = get_image_colour_string(dds.dds_director.mipmap(DDS_MIPMAP_DXT3_DXT5).colourspace)
			cprint using "> DXT3,DXT5  \                        \ \    \"; method; colourspace
		
			method = dds.dds_director.mipmap(DDS_MIPMAP_NORMALMAP).resize_filter
			colourspace = get_image_colour_string(dds.dds_director.mipmap(DDS_MIPMAP_NORMALMAP).colourspace)
			cprint using "> NORMALMAP  \                        \ \    \"; method; colourspace
			
			cprint ""
			cprint "> start the process? (y/n)"
			do
				cprint "> ";
				input kstr
				kstr = lcase(kstr)
			loop while ((kstr <> "y") and (kstr <> "n"))
			cprint ""
			if (kstr = "n") then
				exit_main(0)
			endif
			
			#if _DEBUG = 1
				start_time = timer
			#endif
			
			cprint "rebuilding peg_pc and g_peg_pc files..."
			n = peg_pc.rebuild_peg(srcfilename, "")
			if (n = F_ERROR) then
				cprint "Error: Fail to rebuild."
				chdir(currentdir)
				exit_main(n)
			endif

			#if _DEBUG = 1
				end_time = timer
				cprint "Elapsed time = "; end_time - start_time
			#endif

		case "g_peg_pc"	'g_peg_pc viewer
			n = g_peg_pc_viewer(srcfilename)
			close
			chdir(currentdir)
			return(n)
			
		case "dds"			'dds file viewer
			n = dds_file_viewer(srcfilename)
			close
			chdir(currentdir)
			return(n)

		case "png"	'convert to dds or tgn
			cprint "> Input conversion type."
			cprint "> 1:png to dds 2:normalmap to tgn 0:Exit"

			do
				cprint "> ";
				input kstr
			loop until kstr = "0" or kstr = "1" or kstr = "2"

			select case kstr
				case "0"
					close
					chdir(currentdir)
					return(F_NOERROR)
				case "1"
					n = convert_png_to_dds(srcfilename)
					if (n = F_ERROR) then
						exit_main(F_ERROR)
					else
						close
						chdir(currentdir)
						return(n)
					endif
				case "2"
					n = convert_png_to_tgn(srcfilename)
					if (n = F_ERROR) then
						chdir(currentdir)
						exit_main(n)
					endif
				case else
			end select

		case else
			cprint "Error:Unsupported file."
			chdir(currentdir)
			exit_main(F_ERROR)

	end select
	
	if (dds.dds_director.silentmode) then
		close
		chdir(currentdir)
		return F_NOERROR
	else
		chdir(currentdir)
		exit_main(F_NOERROR)
	endif

end function

'--------------------------------------------------------------------
'	wait keypress
'--------------------------------------------------------------------
function wait_anykey() as string
	dim kstr as string
	do
		kstr = inkey
	loop while kstr = ""
	return kstr
end function

'--------------------------------------------------------------------
'	view textures in *.gpeg_pc file
'--------------------------------------------------------------------
function g_peg_pc_viewer(srcfilename as string) as s32
	dim as string peg_filename, gpeg_filename, title, kstr
	dim as s32 n, t, entry, imgw, imgh
	dim mouse as mouse_data
	dim pegheader as peg_pc.peg_header
	dim image_rgba(any) as u8
	redim names(any) as string
	redim frames(any) as peg_pc.peg_frame

	peg_filename = libfio.change_filename_ext(srcfilename, ".peg_pc")
	gpeg_filename = srcfilename

	cprint "View texture in [ ";peg_pc.get_pegname(srcfilename); ".g_peg_pc ]."
	
	n = peg_pc.get_pegdata(peg_filename, pegheader, names(), frames())
	if (n = F_ERROR) then
		cprint "Error: Fail to get peg file data."
		return F_ERROR
	end if
	if (pegheader.entrycount < 1) then
		cprint "Error: no entrity."
		return F_ERROR		
	endif
	cprint "EntryCount = "; 
	cprint "TotalFrame = "; pegheader.framecount
	cprint ""	
	cprint "entry + : mouse wheel+ or UpArrow key"
	cprint "entry - : mouse wheel- or DownArrow key"
	cprint "exit    : close window"
	cprint "          or press ESC."
	cprint ">"

	entry = 0
	do
		imgw = frames(entry).width
		imgh = frames(entry).height
		n = peg_pc.get_image_gpeg(gpeg_filename, image_rgba(), pegheader, frames(), entry)
		if (n = F_ERROR) then
			cprint "Error: Fail to get texture image."
			return(n)
		end if
		title = str(entry + 1) + "/" + str(pegheader.entrycount)
		select case frames(entry).format
			case D3DFMT_A8R8G8B8
				title += " A8R8G8B8"
			case D3DFMT_R5G6B5
				title += " R5G6B5"
			case D3DFMT_DXT1
				title += " DXT1"
			case D3DFMT_DXT3
				title += " DXT3"
			case D3DFMT_DXT5
				title += " DXT5"
			case else
		end select
		title += (" mip:" + str(cint(frames(entry).mipmapcount)))
		graphics.draw_image(title, image_rgba(), imgw, imgh)
		do
			do
				n = getmouse(mouse.x, mouse.y, mouse.wheel, mouse.button, mouse.clip)
				kstr = inkey()
				if (kstr <> "") then
					exit do
				endif
			loop while (n)
			if ((mouse.wheel > 0) or (kstr = KEY_UPARROW)) then
				entry += 1
				if (entry > pegheader.entrycount - 1) then
					entry = 0
				end if
				exit do					
			endif
			if ((mouse.wheel < 0) or (kstr = KEY_DOWNARROW)) then
				entry -= 1
				if (entry < 0) then
					entry = pegheader.entrycount - 1
				end if
				exit do
			end if		
			if ((kstr = KEY_ESC) or (kstr = GRAPHICS_WINDOW_CLOSED)) then
				graphics.close_window()
				exit do
			endif
		loop while (true)
		if ((kstr = KEY_ESC) or (kstr = GRAPHICS_WINDOW_CLOSED)) then
			exit do
		endif
	loop while (true)
	return F_NOERROR
end function

'--------------------------------------------------------------------
'	dds file viewer
'--------------------------------------------------------------------
function dds_file_viewer(srcfilename as string) as s32
	dim as s32 mipmap, imgw, imgh, fmt
	dim as string title, kstr
	
	cprint "DDS file viewer."
	cprint "Close window to exit."
	cprint "Mouse left click on image to change background type."
	mipmap = dds.load_dds(srcfilename, squish.image_rgba(), imgw, imgh, fmt)
	if (mipmap > 0) then
		select case fmt
			case D3DFMT_A8R8G8B8
				title = "A8R8G8B8 "+str(imgw)+"x"+str(imgh)
			case D3DFMT_R5G6B5
				title = "R5G6B5 "+str(imgw)+"x"+str(imgh)
			case D3DFMT_DXT1
				title = "DXT1 "+str(imgw)+"x"+str(imgh)+" mipmap:"+str(mipmap) 
			case D3DFMT_DXT3
				title = "DXT3 "+str(imgw)+"x"+str(imgh)+" mipmap:"+str(mipmap) 
			case D3DFMT_DXT5
				title = "DXT5 "+str(imgw)+"x"+str(imgh)+" mipmap:"+str(mipmap) 
			case else
				cprint "Error:Unsupported format:";fmt
				return F_ERROR
		end select
		'	draw image
		graphics.draw_dds_blocks(title, squish.image_rgba(), imgw, imgh, fmt, mipmap)
	endif
	return F_NOERROR
end function

'--------------------------------------------------------------------
'	input integer (restricted within min to max)
'--------------------------------------------------------------------
function select_number(minval as s32, maxval as s32) as s32
	dim as s32 n, cx, cy
	cga.get_cursor(cx, cy)
	do
		cga.set_cursor_state(CGA_CURSOR_HIDE)
		cga.set_cursor(cx, cy)
		cprint spc(79);
		cga.set_cursor(cx, cy)
				
		cprint "> ";
		input n
	loop while ((n < minval) or (n > maxval))
	return n	
end function


'--------------------------------------------------------------------
'	input double with default value
'--------------------------------------------------------------------
function input_number(k as double, fms as string, minv as double, maxv as double) as double
	dim as s32 sx, sy, cx, cy, x, lg
	dim as string kstr, nst, lst, rst
	dim as double v

	nst = format(k, fms)

	lg = len(nst)
	x = lg
	cga.get_cursor(sx, sy)
	cprint nst;
	cga.set_cursor(sx + x - 1, sy)
	cga.set_cursor_state(CGA_CURSOR_BLINK)
	do
		kstr = wait_anykey()		
		select case kstr
			case KEY_LEFTARROW
				if (x > 1) then
					x-=1
					cga.set_cursor(sx + x - 1, sy)
				endif
			case KEY_RIGHTARROW
				if (x < lg) then 
					x+=1
					cga.set_cursor(sx + x - 1, sy)
				endif
			case KEY_DEL
				select case mid(nst, x, 1)
					case "0" to "9"
						mid(nst, x, 1) = "0"
					case else
						'nop
				end select
			case KEY_BS
				if (x > 1) then
					select case mid(nst, x - 1, 1)
						case "0" to "9"
							mid(nst, x - 1, 1) = "0"
						case else
							'nop
					end select
					x-=1
				endif
			case "-"
				select case mid(nst, x, 1)
					case " "
						mid(nst, x, 1) = kstr
					case "-"
						mid(nst, x, 1) = " "						
					case else
						'nop
				end select
			case "."
				if (mid(nst, x, 1) = ".") then
					if (x < lg) then x+=1
				endif
			case "0" to "9"
				select case mid(nst, x, 1)
					case "0" to "9"
						mid(nst, x, 1) = kstr
						if (x < lg) then x+=1
					case else
						'nop
				end select
			case KEY_ENTER
				v = val(nst)
				if ((v >= minv) and (v <= maxv)) then
					cga.set_cursor(0, sy + 1)		'	CR/LF
					exit do
				endif
			case else
		end select
		cga.set_cursor_state(CGA_CURSOR_HIDE)
		cga.set_cursor(sx, sy)
		cprint spc(79);
		cga.set_cursor(sx, sy)
		cprint nst;
		cga.set_cursor(sx + x - 1, sy)
		cga.set_cursor_state(CGA_CURSOR_BLINK)
	loop while true

	return v
	
end function

'--------------------------------------------------------------------
'	convert png to dds 
'--------------------------------------------------------------------
function convert_png_to_dds(srcfilename as string) as s32
	dim as s32 i, n, imgw, imgh, bytepp, fmt, normalmap, texture_format
	dim as string ddsfile, title, kstr
	dim as double k
	dim image_rgba(any) as u8

	n = libpng.load_png(srcfilename, image_rgba(), imgw, imgh, bytepp)
	if (n <> PNG_NO_ERROR) then
		cprint "Error: Fail to load png file."
		return F_ERROR
	endif

	'	compression method
	cprint "> Input compression method."
	cprint "  1:DXT1 3:DXT3 5:DXT5 a:Auto 0:Exit"
	do
		cprint "> ";
		input kstr
	loop until kstr = "a" or kstr = "1" or kstr = "3" or kstr = "5" or kstr = "0"
	select case lcase(kstr)
		case "a"
			fmt = 0
		case "1"
			fmt = D3DFMT_DXT1
		case "3"
			fmt = D3DFMT_DXT3
		case "5"
			fmt = D3DFMT_DXT5
		case "0"
			return F_NOERROR
		case else
	end select

	if (fmt = 0) then					
		cprint "Analyzing image to get suitable compression method. Wait a moment..."
		fmt = dds.get_suitable_format(image_rgba(), imgw, imgh)
		select case fmt
			case D3DFMT_DXT1
				cprint "DXT1 is most suitable for this image."
			case D3DFMT_DXT3
				cprint "DXT3 is most suitable for this image."
			case D3DFMT_DXT5
				cprint "DXT5 is most suitable for this image."
			case else
		end select
	endif

	select case fmt
		case D3DFMT_DXT1
			texture_format = DDS_MIPMAP_DXT1
		case D3DFMT_DXT3, D3DFMT_DXT5
			texture_format = DDS_MIPMAP_DXT3_DXT5
		case else		
	end select

	'	check the texture is normalmap/tgn or not
	normalmap = dds.is_normalmap(image_rgba(), imgw, imgh, fmt)

	if (not(normalmap)) then
		'	select resize filter
		cprint "> Resize Filter."
		cprint "  1:LINEAR"
		cprint "  2:CUBIC"
		cprint "  3:FANT"
		cprint "  4:BOX"
		cprint "  5:TRIANGLE"
		cprint "  0:Exit"
		n = select_number(0, 10)
		select case n
			case 0
				close
				return F_NOERROR							
			case 1
				dds.dds_director.mipmap(texture_format).resize_filter = "LINEAR"
			case 2
				dds.dds_director.mipmap(texture_format).resize_filter = "CUBIC"
			case 3
				dds.dds_director.mipmap(texture_format).resize_filter = "FANT"
			case 4
				dds.dds_director.mipmap(texture_format).resize_filter = "BOX"
			case 5
				dds.dds_director.mipmap(texture_format).resize_filter = "TRIANGLE"
			case else
		end select
		
		cprint "> 1:Filter without dithering 2:Filter with dithering 0:Exit"
		n = select_number(0, 2)
		select case n
			case 0
				close
				return F_NOERROR
			case 2
				dds.dds_director.mipmap(texture_format).resize_filter += "_DITHER"
			case else
		end select
		
		if (n = 2) then
			cprint "> 1:Filter without diffusion 2:Filter with diffusion 0:Exit"
			n = select_number(0, 2)
			select case n
				case 0
					close
					return F_NOERROR
				case 2
					dds.dds_director.mipmap(texture_format).resize_filter += "_DIFFUSION"
				case else
			end select
		endif
	endif
		
	title = PNGTITLESTRING(imgw, imgh, bytepp)
	graphics.draw_image(title, image_rgba(), imgw, imgh)

	cprint "Converting image to dds format. Wait a moment..."
	
	ddsfile = libfio.change_filename_ext(srcfilename, ".dds")
	
	n = dds.save_dds_rgba(ddsfile, image_rgba(), imgw, imgh, fmt, 1)

	graphics.close_window()
	
	if (n = 0) then 
		cprint "Error:convert to dds file failed."
		return F_ERROR
	else
		cprint "DDS file was generated successfully."
		cprint using "elapsed time ###.#### sec."; dds.dds_director.elapsed_time
		cprint ""
		n = dds_file_viewer(ddsfile)
		return n
	endif
end function

'--------------------------------------------------------------------
'	convert normalmap to tgn(image with alpha)
'--------------------------------------------------------------------
function convert_png_to_tgn(srcfilename as string) as s32
	dim as s32 n, imgw, imgh, bytepp, flag
	dim as string tgnfilename, title
	dim image_rgba(any) as u8
	dim imagebug_rgba(any) as u8
	
	tgnfilename = libfio.change_filename_ext(srcfilename, "_tgn.png")
	n = libpng.load_png(srcfilename, image_rgba(), imgw, imgh, bytepp)

	if (n <> PNG_NO_ERROR) then
		cprint "Error: Fail to load png file."
		return F_ERROR
	else
		title = PNGTITLESTRING(imgw, imgh, bytepp)
		n = imgw * imgh * 4
		redim imagebuf_rgba(n - 1) as u8
		dds.normalmap_to_tgn(image_rgba(), imagebuf_rgba(), imgw, imgh)
		graphics.draw_image(title, imagebuf_rgba(), imgw, imgh)
		n = libpng.save_png(tgnfilename, imagebuf_rgba(), imgw, imgh, 32)
	end if
	return F_NOERROR
end function

'--------------------------------------------------------------------
'	readout settings from sr2pegtool.ini (xml format) 
'--------------------------------------------------------------------
function read_ini(filename as string) as s32
	dim as s32 i, n, ret, texture_format
	dim as string ename, evalue, toolpath, temppath

	'	open xml file		
	ret = xml.read_doc(filename)
	if (ret <> XML_NO_ERROR) then
		cprint using "Error:Can't read & file."; _INIFILE_NAME
		xml.close_doc()
		return(F_ERROR)
	endif

	'	<sr2pegtool>
	ret = xml.read_element(ename, evalue)
	if (ename <> "sr2pegtool") then
		cprint using "Error:& <sr2texturetool> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	endif

	'	<Version>
	ret = xml.read_element(ename, evalue)
	if (ename <> "Version") then
		cprint using "Error:& <Version> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	elseif (evalue <> PROGRAM_VERSION) then
		cprint using "Error:& <Version> value not match. <&>"; _INIFILE_NAME; evalue
	endif

	'	<PathSettings>
	ret = xml.read_element(ename, evalue)
	if (ename <> "PathSettings") then
		cprint using "Error:& <PathSettings> missing."; _INIFILE_NAME
		xml.close_doc()
		return(F_ERROR)
	endif
	do
		ret = xml.read_element(ename, evalue)
		select case ename
			case "/PathSettings"
				exit do
		end select
	loop while (true)

	'	<PegDirector>
	ret = xml.read_element(ename, evalue)
	if (ename <> "PegDirector") then
		cprint using "Error:& <PegDirector> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	endif
	
	do
		ret = xml.read_element(ename, evalue)
		select case ename
			case "CheckImageSizeDiff"
				select case evalue
					case "TRUE"
						peg_pc.peg_director.check_imagesize_difference = true
					case "FALSE"
						peg_pc.peg_director.check_imagesize_difference = false
					case else						
						cprint using "Error:& <CheckImageSizeDiff> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "Suppress_extracted"
				select case evalue
					case "TRUE"
						peg_pc.peg_director.suppress_extracted = true
					case "FALSE"
						peg_pc.peg_director.suppress_extracted = false
					case else						
						cprint using "Error:& <Suppress_extracted> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "/PegDirector"
				exit do
			case else
				cprint using "Error:& <PegDirector> block syntax error. <&>"; _INIFILE_NAME; ename
				xml.close_doc()
				return(F_ERROR)
		end select
	loop while (true)
	
	'	<DDSDirector>
	ret = xml.read_element(ename, evalue)
	if (ename <> "DDSDirector") then
		cprint using "Error:& <DDSDirector> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	endif
	
	'	<Const>
	ret = xml.read_element(ename, evalue)	'Const
	if (ename <> "Const") then
		cprint using "Error:& <Const> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	endif
	
	do
		ret = xml.read_element(ename, evalue)
		select case ename
			case "RenderTarget"
				select case evalue
					case "SR2"
						dds.dds_director.render_target = DDS_SR2_ENGINE
					case "GENERIC"
						dds.dds_director.render_target = DDS_GENERIC_ENGINE
					case else
						cprint using "Error:& <RenderTarget> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "SilentMode"
				select case evalue
					case "TRUE"
						dds.dds_director.silentmode = true
					case "FALSE"
						dds.dds_director.silentmode = false
					case else						
						cprint using "Error:& <SilentMode> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "/Const"
				exit do
			case else
				cprint using "Error:& <Const> block syntax error. <&>"; _INIFILE_NAME; ename
				xml.close_doc()
				return(F_ERROR)
		end select
	loop while (true)

	'	<DxtDirector>
	ret = xml.read_element(ename, evalue)	'Const
	if (ename <> "DxtDirector") then
		cprint using "Error:& <DxtDirector> missing."; _INIFILE_NAME
		xml.close_doc()
		return(F_ERROR)
	endif
	do
		ret = xml.read_element(ename, evalue)
		select case ename
			case "ColourFit"
				select case evalue
					case "ClusterFit"
						dds.dds_director.colourfit = squish_kColourClusterFit
					case "IterativeClusterFit"
						dds.dds_director.colourfit = squish_kColourIterativeClusterFit
					case "RangeFit"
						dds.dds_director.colourfit = squish_kColourRangeFit
					case else
						cprint using "Error:& wrong description in <ColourFit>. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "/DxtDirector"
				exit do
			case else
				cprint using "Error:& <DxtDirector> block syntax error. <&>"; _INIFILE_NAME; ename
				xml.close_doc()
				return(F_ERROR)
		end select
	loop while (true)

	'	<MipmapDirectors>
	ret = xml.read_element(ename, evalue)
	if (ename <> "MipmapDirectors") then
		cprint using "Error:& <MipmapDirectors> missing. <&>"; _INIFILE_NAME; ename
		xml.close_doc()
		return(F_ERROR)
	endif

	do
		ret = xml.read_element(ename, evalue)
		select case ename
			case "MipmapDirector"
				'	nop
			case "Format"
				select case evalue
					case "NORMALMAP"
						texture_format = DDS_MIPMAP_NORMALMAP
					case "DXT1"
						texture_format = DDS_MIPMAP_DXT1
					case "DXT3orDXT5"
						texture_format = DDS_MIPMAP_DXT3_DXT5
					case else
						cprint using "Error:& <Format> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "Filter"
				dds.dds_director.mipmap(texture_format).resize_filter = trim(evalue)
			case "Colour"
				select case evalue
					case "sRGB"
						dds.dds_director.mipmap(texture_format).colourspace = DDS_sRGB_COLOUR
					case "LINEAR"
						dds.dds_director.mipmap(texture_format).colourspace = DDS_LINEAR_COLOUR
					case else	
						cprint using "Error:& <Colour> value wrong. <&>"; _INIFILE_NAME; evalue
						xml.close_doc()
						return(F_ERROR)
				end select
			case "/MipmapDirector"
				'	nop
			case "/MipmapDirectors"
				exit do
			case else
				cprint using "Error:& <MipmapDirectors> block syntax error. <&>"; _INIFILE_NAME; ename
				xml.close_doc()
				return(F_ERROR)
		end select
	loop while (true)

	xml.read_element(ename, evalue)	' </DDSDirector>
	xml.read_element(ename, evalue)	' </sr2pegtool>
	xml.close_doc()	
	
	return(F_NOERROR)
end function

'--------------------------------------------------------------------
'	get image colour as string
'--------------------------------------------------------------------
function get_image_colour_string(n as s32) as string
	select case n
		case DDS_sRGB_COLOUR
			return("sRGB")
		case DDS_LINEAR_COLOUR
			return("LINEAR")
		case else
			return("Undefined")
	end select
end function

end( main() )

